package com.diagbot.filter;

import com.diagbot.client.DiagbotmanServiceClient;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.RespDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.exception.ErrorCode;
import com.diagbot.util.GsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;

/**
 * @Description: 全局过滤器
 * @author: gaodm
 * @time: 2018/9/7 13:55
 */
@Configuration
@Slf4j
public class GlobalGatewayFilter implements GlobalFilter {

    private static final String GATE_WAY_PREFIX = "/api";
    private static final Boolean IS_GENERATE = false;
    private static List<String> SERVICE_FILTER = new ArrayList<>();

    @Autowired
    UserServiceClient userServiceClient;
    @Autowired
    DiagbotmanServiceClient diagbotmanServiceClient;


    @Override
    public Mono<Void> filter(ServerWebExchange serverWebExchange, GatewayFilterChain gatewayFilterChain) {
        log.info("check token and url permission....");
        LinkedHashSet requiredAttribute = serverWebExchange.getRequiredAttribute(ServerWebExchangeUtils.GATEWAY_ORIGINAL_REQUEST_URL_ATTR);
        ServerHttpRequest request = serverWebExchange.getRequest();
        String requestUri = request.getPath().pathWithinApplication().value();
        String serviceName = "";
        if (requiredAttribute != null) {
            Iterator<URI> iterator = requiredAttribute.iterator();
            while (iterator.hasNext()){
                URI next = iterator.next();
                if(next.getPath().startsWith(GATE_WAY_PREFIX)){
                    requestUri = next.getPath().substring(GATE_WAY_PREFIX.length());
                    serviceName = requestUri.substring(requestUri.indexOf('/') + 1, requestUri.indexOf('/', 2));
                }
            }
        }
        log.info("APIURL:{}", requestUri);
        log.info("SERVICENAME:{}", serviceName);
        //

//        if(!IS_GENERATE) {
//            SERVICE_FILTER = diagbotmanServiceClient.getAll().data;
//        }
//        if(SERVICE_FILTER.contains(serviceName)) {
//            String appkey = request.getHeaders().getFirst("appkey");
//            String secret = request.getHeaders().getFirst("secret");
//            //获取appkey，secret对应的
//            RespDTO<Permission> res = userServiceClient.getPermission(appkey, secret);
//        }
//        if(1 == 1) {
//            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "出错了!");
//        }

//        if(1==1){
//            return getVoidMono(serverWebExchange, CommonErrorCode.NO_PERMISSION);
//        }

        ServerHttpRequest.Builder builder = serverWebExchange.getRequest().mutate();
        builder.header("Authorization","Authorization Bearer token");
        gatewayFilterChain.filter(serverWebExchange.mutate().request(builder.build()).build());
        return gatewayFilterChain.filter(serverWebExchange.mutate().request(builder.build()).build());
    }

    /**
     * 网关抛异常
     *
     * @param body
     */
    private Mono<Void> getVoidMono(ServerWebExchange serverWebExchange, ErrorCode body) {
        RespDTO resp = new RespDTO();
        serverWebExchange.getResponse().setStatusCode(HttpStatus.OK);
        resp.msg = body.getMsg();
        resp.code = body.getCode();
        resp.data = "";
        byte[] bytes = GsonUtil.toJson(resp).getBytes(StandardCharsets.UTF_8);
        DataBuffer buffer = serverWebExchange.getResponse().bufferFactory().wrap(bytes);
        return serverWebExchange.getResponse().writeWith(Flux.just(buffer));
    }
}
